"""game"""

import random
import time
# import user_interface.user_interface as ui
import clean_input as ci


def check_special_score(score, p):
    """
    Checks if player of AI have a score of 9, 10 or >=16.
    If a score is >=16, 'restart' gets returned and the game
    is immediately over.
    Otherwise the score is returned.

    False == break for loop
    True == do nothing

    :param p: tuple with current player
    :param score:
    :return: bool, 'restart' or score
    """

    if score == 9 and p[1]:  # player has 9
        return True, score
    elif score == 10 and p[1]:  # player has 10
        print("Sie haben 10 erreicht. Noch ein letzter Wurf...")
        time.sleep(1)
        r = roll_dice()
        print(f"Sie würfeln {r}.")
        if score + sum(r) == 16:
            print(f"Sie haben insgesamt {score} gewürfelt und damit verloren.\n")
            return False, "restart"
        return True, score + sum(r)
    elif score >= 16 and p[1]:  # player has too much
        print(f"Sie haben insgesamt {score} gewürfelt und damit verloren.\n")
        return False, "restart"
    elif score == 9 and not p[1]:  # ai has 9
        return True, score
    elif score == 10 and not p[1]:  # ai has 10
        print(f"{p[0]} hat insgesamt 10 gewürfelt und muss noch einmal würfeln.")
        r = roll_dice()
        print(f"{p[0]} würfelt {r}.")
        if score + sum(r) == 16:
            print(f"{p[0]} hat insgesamt {score + sum(r)} gewürfelt und damit verloren.\n")
            return False, "restart"
        return True, score + sum(r)
    elif score >= 16 and not p[1]:  # ai has too much
        print(f"{p[0]} hat insgesamt {score} gewürfelt und damit verloren.\n")
        return False, "restart"
    else:
        return False, score


def choose_loser(all_results):
    """
    Determines the players with the lowest score
    and prints them.
    Then starts a new game.

    :param all_results: list of dicts with player and final score
    :return:
    """
    loser_score = 15
    losers = []

    for r in all_results:  # get lowest score
        if r["final_score"] < loser_score:
            loser_score = r["final_score"]

    for r in all_results:  # check if multiple players with lowest score
        if r["final_score"] == loser_score:
            losers.append(r["player"])

    loser_string = "Die folgenden Spieler haben verloren: "
    losers = ",".join(losers)
    loser_string = loser_string + losers

    print(loser_string)

    main()


def roll_dice(number=1, faces=6, seed=None):
    """
    Throws a certain number of dice with a certain number of faces.

    :param number: Number of dice
    :param faces: Number of faces of a die
    :param seed: Number for random.seed() to reproduce randomness
    :return: List of results
    """
    if seed:
        random.seed(seed)

    results = []

    for n in range(number):
        #  ui.dice_animation()
        result = random.randint(1, faces)
        results.append(result)

    return results


def sixteen_is_dead(players):
    """
    Most important method.
    Rotates over all players and keeps track of their score.
    If the human player presses 'n', the dice get handed to the AI.

    :param players: list of tuples
    :return:
    """
    all_results = []

    for p in players:

        if p[1]:  # human player
            button = input("Sie sind an der Reihe!")
            player_score = 0

            while button != "n":
                current_roll = roll_dice()
                print("Sie haben " + str(current_roll) + " gewürfelt!")

                player_score += sum(current_roll)

                button, player_score = check_special_score(player_score, p)

                if not button and player_score == "restart":
                    main()

                print("Damit haben Sie insgesamt " + str(player_score) + " gewürfelt.")

                # if player_score >= 16:
                #     print("Sie haben verloren!")
                #     sys.exit()

                if button:
                    break

                button = ci.io("Wollen Sie weiterwürfeln?  ",["", "n"])
                # button = input("Wollen Sie weiterwürfeln?")

            all_results.append({"player": p[0], "final_score": player_score})

        else:  # AI
            ai_score = 0

            while ai_score < 13:  # roll until score at least 12
                current_roll = roll_dice()
                print(f"{p[0]} würfelt {current_roll}.")

                ai_score += sum(current_roll)

                button, ai_score = check_special_score(ai_score, p)

                if not button and ai_score == "restart":
                    main()

                print(f"{p[0]} hat damit insgesamt {ai_score} gewürfelt.")

                if button:
                    break

            all_results.append({"player": p[0], "final_score": ai_score})
    else:
        print(all_results)
        choose_loser(all_results)


def main():
    """
    Assigns human player name and number of players.
    Construct list of tuples with players.

    :return:
    """
    with open("player_welcome_message", "r", encoding="utf8") as f:
        welcome = ""
        for l in f.readlines():
            welcome += l
    # print(welcome)
    eval(welcome)

    name = ""
    while not name:
        name = input("Wie heißen Sie?  ")
    print(f"Willkommen, {name}!")

    player_number = 0
    while not player_number:
        try:
            player_number = int(input("Gegen wie viele Personen wollen Sie spielen?  "))
        except:
            print("Ihre Eingabe war nicht korrekt. Bitte versuchen Sie es nochmal.  ")
    #  TODO: player_number check for integer

    players = []
    human = (name, True)
    players.append(human)

    for p in range(int(player_number)):
        players.append(("P" + str(p), False))

    sixteen_is_dead(players)


if __name__ == "__main__":
    main()
